package conferenciaJPAAlunos;

import java.util.*;
import javax.persistence.*;

@Entity
@Table(name="ARTIGO")

public class Artigo {
  
    private String nome;
    private String resumo;
    private int identificador;
    private String nomeArquivo;
    private double notaFinal;
    private int status;
    private boolean temRevisao;

    private Set<Autor> autores;
    private Set<Avaliador> avaliadoresConflitantes; //tem conflito com avaliadores
    private Set<Topico> topicos;
    private Set<Revisao> revisoes;
	    

    public Artigo (){
    	this.status = 0; //por default e' nao aceito
    	this.autores = new HashSet<Autor>();
    	this.avaliadoresConflitantes = new HashSet<Avaliador> ();
    	this.revisoes = new HashSet<Revisao> ();
    	this.topicos = new HashSet<Topico> ();  	
    	this.temRevisao = false;
    }

	public int getIdentificador() {
		return identificador;
	}

	@Id
	public String getNome() {
		return nome;
	}

	@Column(name="NOME_ARQUIVO")
	public String getNomeArquivo() {
		return nomeArquivo;
	}

	public double getNotaFinal() {
		return notaFinal;
	}

	public String getResumo() {
		return resumo;
	}

	public int getStatus() {
		return status;
	}

	@ManyToMany
	@JoinTable(name="ARTIGO_AUTOR",joinColumns=@JoinColumn(name="NOME_ARTIGO",referencedColumnName="NOME"),inverseJoinColumns=@JoinColumn(name="NOME_AUTOR",referencedColumnName="NOME")) 
	public Set<Autor> getAutores() {
		return autores;
	}

	@ManyToMany
	@JoinTable(name="ARTIGO_AVALIADORCONFLITANTE",joinColumns=@JoinColumn(name="NOME_ARTIGO",referencedColumnName="NOME"),inverseJoinColumns=@JoinColumn(name="NOME_AVALIADOR",referencedColumnName="NOME"))
	public Set<Avaliador> getAvaliadoresConflitantes() {
		return avaliadoresConflitantes;
	}
	
	@OneToMany(cascade=CascadeType.ALL, mappedBy="artigo")
	public Set<Revisao> getRevisoes() {
		return revisoes;
	}

	public Set<Topico> getTopicos() {
		return topicos;
	}

	public void setIdentificador(int identificador) {
		this.identificador = identificador;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNomeArquivo(String nomeArquivo) {
		this.nomeArquivo = nomeArquivo;
	}

	public void setNotaFinal(double notaFinal) {
		this.notaFinal = notaFinal;
	}

	public void setResumo(String resumo) {
		this.resumo = resumo;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	@Column(name="TEM_REVISAO")
	public boolean isTemRevisao() {
		return temRevisao;
	}

	public void setTemRevisao(boolean temRevisao) {
		this.temRevisao = temRevisao;
	}

	public void setAutores(Set<Autor> list) {
		autores = list;
	}

	public void setAvaliadoresConflitantes(Set<Avaliador> list) {
		avaliadoresConflitantes = list;
	}

	public void setRevisoes(Set<Revisao> list) {
		revisoes = list;
	}

	public void setTopicos(Set<Topico> list) {
		topicos = list;
	}

    public void associaAutor(Autor autor){
    	if (!this.autores.contains(autor))
	    	this.autores.add(autor);
    }

    public void associaTopico(Topico topico){
    	if (!this.topicos.contains(topico))
	    	this.topicos.add(topico);
    }

    public void defineConflito(Avaliador avaliador){
    	if (!this.avaliadoresConflitantes.contains(avaliador))
	    	this.avaliadoresConflitantes.add(avaliador);
    }

    public void associaRevisao(Revisao revisao){
    	this.temRevisao = true;
    	if (this.revisoes.size() < 3 && !this.revisoes.contains(revisao))
    		this.revisoes.add(revisao);
    }

    public void calculaNotaFinal(){
		double soma = 0;
    	Iterator<Revisao> iter = this.revisoes.iterator();
    	while (iter.hasNext())
    		soma = soma + iter.next().getNotaFinal();
    	this.notaFinal = soma / this.revisoes.size();
    }

    public void defineComoAceito(){
    	this.status = 1;
    }

    public void modificaStatus(){
    	if (this.status == 1)
    		this.status = 0;
    	else
    		this.status = 1;	
    }
    
	public String toString (){
		return this.nome + " " + this.identificador + " " + this.status;
	}
}
