package conferenciaJPAAlunos;

import java.sql.*;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import javax.persistence.*;

@Entity
@Table(name="CONFERENCIA")

public class Conferencia {
	
	private static Conferencia conferencia;
	
	private static EntityManagerFactory factory;
	private static EntityManager em;

	
	private boolean iniciada;
	
    private String nome;
    private String local;
    private Date data;
    private int numMaxArtigos;
    private Date dataInicioSubmissao;
    private Date dataTerminoSubmissao;
    private Date dataTerminoRevisao;

	public static Conferencia getInstance (){
		if (conferencia == null)
			conferencia = new Conferencia (true);
		return conferencia;	 
	}
	
	public Conferencia() {
		this(false);
	}

    private Conferencia (boolean init) {
    	if (!init) return;
		if (conferencia != null) {
			throw new RuntimeException("conferencia eh um singleton!");
		}
		try{
			Conferencia.factory = Persistence.createEntityManagerFactory("conferencia_toplink"); // utiliza o TopLink-JPA - ver META-INF/persistence.xml para detalhes sobre esta configuracao
			Conferencia.em = Conferencia.factory.createEntityManager();
			List list = Conferencia.em.createQuery("select a from Conferencia a").getResultList();
			this.iniciada = list.size()>0;
		}
		catch (Exception e){
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao configurar o mapeador");		
		}

	}

    public void iniciaConferencia (String nome, String local, Date data, 
            int numMaxArtigos, Date dataInicioSubmissao, Date dataTerminoSubmissao, 
            Date dataTerminoRevisao, String[] nomesTopicos) {
		if (!this.isIniciada()){			
	    	this.nome = nome;
	    	this.local = local;
	    	this.data = data;
	    	this.numMaxArtigos = numMaxArtigos;
	    	this.dataInicioSubmissao = dataInicioSubmissao;
	    	this.dataTerminoSubmissao = dataTerminoSubmissao;
	    	this.dataTerminoRevisao = dataTerminoRevisao;
	    	try {
	    		em.getTransaction().begin();
	    		em.persist(this);
				this.iniciada = true;            	
		    	for (int i=0; i<nomesTopicos.length; i++){		
		    		Topico topico = new Topico ();
		    		topico.setNome((String)nomesTopicos[i]);
		    		em.persist(topico);
		    	}
		    	em.getTransaction().commit();
	    	} catch (Exception e) {
	    		e.printStackTrace();
	    		throw new java.lang.RuntimeException("erro ao inserir conferencia");
	    	}
		}
    }
    
    @Transient
    public boolean isIniciada(){
    	return this.iniciada;
    }
    
    public void insereTopico (String nome){
   		Topico topico = new Topico ();
   		topico.setNome(nome);
    	try {
    		em.getTransaction().begin();
    		em.persist(topico);
    		em.getTransaction().commit();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao inserir topico");
    	}	
    }

    public void defineChefe(String nome, String login, String senha){
    	try{
        	ChefeComite.inicializaClasseChefeComite(nome);
    		em.getTransaction().begin();
    		ChefeComite chefe = em.find(ChefeComite.class,nome);
    		if (chefe == null){
		    	chefe = new ChefeComite();
    			chefe.setNome(nome);
    			chefe.setLogin(login);
    			chefe.setSenha(senha);
        		em.persist(chefe);
    		}	
    		em.getTransaction().commit();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao inserir chefe");
    	}
    }

    public void insereAvaliador(String nome, String login, String senha, 
            String email, String origem, String[] nomesTopicos){
		try {
			em.getTransaction().begin();
			Avaliador avaliador = em.find(Avaliador.class,nome);
	    	if (avaliador == null) {
	    		avaliador = new Avaliador();
	    	}
			avaliador.setNome(nome);
			avaliador.setLogin(login);
			avaliador.setSenha(senha);
			avaliador.setEmail(email);
			avaliador.setOrigem(origem);
			for (int i=0; i<nomesTopicos.length; i++){
				String nomeTopico = (String)nomesTopicos[i];	
				Topico topico = em.find(Topico.class,nomeTopico);
				if (topico != null) {
			  		avaliador.associaTopico(topico);
				}		
			}	
			em.persist(avaliador);
			  //Enviar email para o avaliador 	
			em.getTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			throw new java.lang.RuntimeException("erro ao inserir avaliador");
		}
	}

    public void mandaEmail(){}
    
    private int nextIdentificador()throws Exception {
		List list = em.createQuery("select max(a.identificador) from Artigo a").getResultList();
    	if (list != null && list.size() > 0) {
    		Integer val = (Integer)list.get(0); //list.firstElement();
    		if (val != null)
    			return val.intValue() + 1;
    	}
    	return 1;
    }

    public void submeteArtigo(String nome, String resumo, String nomeArquivo,
 				String[] nomesAutores, String[] instituicoesAutores, String[] nomesTopicos) {
 		try{			
 			em.getTransaction().begin();
 			Artigo artigo = (Artigo) em.find(Artigo.class,nome);			
	 		if (artigo==null){			
		    	artigo = new Artigo();
		    	artigo.setNome(nome);
		    	artigo.setResumo(resumo);
		    	artigo.setIdentificador(this.nextIdentificador());
		    	artigo.setNomeArquivo(nomeArquivo);
		    	for (int i=0; i<nomesAutores.length; i++){
		    		String nomeAutor = (String)nomesAutores[i];
		    		String instituicaoAutor = (String)instituicoesAutores[i];
	    			Autor autor = (Autor)em.find(Autor.class,nomeAutor);
		    		if (autor == null) {
			    		autor = new Autor ();
			    		autor.setNome(nomeAutor);
			    		autor.setInstituicao(instituicaoAutor);
			    		em.persist(autor);
		    		}
		    		artigo.associaAutor(autor);	
		    	}
	    		
		    	for (int i=0; i<nomesTopicos.length; i++){
		    		String nomeTopico = (String)nomesTopicos[i];
		    		Topico topico = (Topico) em.find(Topico.class,nomeTopico);
		    		if (topico != null) {
			    		artigo.associaTopico(topico);
		    		}		
		    	}
	    		em.persist(artigo);
	 		}
    		em.getTransaction().commit();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao inserir artigo");
    	}
    }

    public List<Artigo> selecionaArtigosSemRevisao(){
    	try {
    		return em.createQuery("select a from Artigo a where a.temRevisao = false").getResultList();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar artigos");
    	}
     }
     
    public List<Avaliador> selecionaAvaliadoresSemConflito(String nomeArtigo){
        // Retorna uma lista contendo os avaliadores sem conflito com um artigo.		\
        	try {
        		Artigo artigo = em.find(Artigo.class,nomeArtigo);
    	   		List avaliadoresConflitantes = em.createQuery("select ac from Artigo art join art.avaliadoresConflitantes ac where art.nome = '" + artigo.getNome() + "'").getResultList();
        		List avaliadores = em.createQuery("select a from Avaliador a").getResultList();
    	    	List<Avaliador> avaliadoresSemConflitos = new ArrayList<Avaliador>();
    	    	for (int i=0; i<avaliadores.size(); i++){
    	    		Avaliador avaliador = (Avaliador)avaliadores.get(i);
    	    		Set artigos = avaliador.retornaArtigosAvaliados();
    	    		if (!artigos.contains(artigo) && !avaliadoresConflitantes.contains(avaliador))
    	    			avaliadoresSemConflitos.add(avaliador);
    	    	}
    			return avaliadoresSemConflitos;
        	} catch (Exception e) {
        		e.printStackTrace();
        		throw new java.lang.RuntimeException("erro ao selecionar os avaliadores sem conflito");
        	}
        }

    public void defineConflito(String nomeArtigo, String nomeAvaliador){
    	try {
    		em.getTransaction().begin();
	    	Artigo artigo = em.find(Artigo.class,nomeArtigo);
	    	Avaliador avaliador = em.find(Avaliador.class,nomeAvaliador);
			artigo.defineConflito(avaliador);
			em.getTransaction().commit();
		} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao definir conflito");
    	}
    }

    public void distribuiArtigos(){
    	
    	//DEFINIR
    	
         }
    

    public Artigo selecionaArtigo(String nome){
    	try {
	    	return em.find(Artigo.class,nome);
    	} catch (Exception e) {
    		return null;
    	}
    }

    public void forneceRevisao(String nomeArtigo, String nomeAvaliador, 
       		 double clareza, double relevancia, double corretude, 
             double notaFinal, String comentariosComite, String comentariosAutor) {             	
    	try {
    		em.getTransaction().begin();
    		Avaliador avaliador = em.find(Avaliador.class,nomeAvaliador);
   			Artigo artigo = em.find(Artigo.class,nomeArtigo);
			Set<Revisao> revisoes = artigo.getRevisoes();
			Iterator<Revisao> iter = revisoes.iterator();
			while (iter.hasNext()){
				Revisao revisao = iter.next();
   				if (revisao.getAvaliador().equals(avaliador)){
   					revisao.setClareza(clareza);
   					revisao.setRelevancia(relevancia);
   					revisao.setCorretude(corretude);
   					revisao.setNotaFinal(notaFinal);
   					revisao.setComentariosComite(comentariosComite);
   					revisao.setComentariosAutor(comentariosAutor);
   					em.persist(revisao);
   				}	
   			}
			em.getTransaction().commit();
   		}  catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao fornecer revisao");
    	}
    }
    

    public List<Artigo> selecionaArtigos(){
    // Seleciona todos os artigos
    	try {
    		return em.createQuery("select a from Artigo a").getResultList();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar artigos");
    	}
    }

    public List<Avaliador> selecionaAvaliadores(){
    // Seleciona todos os avaliadores
    	try {
    		return em.createQuery("select a from Avaliador a").getResultList();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar avaliadores");
    	}
    }
    
    public List<Artigo> selecionaArtigosComNotas(){
    	try {
    		em.getTransaction().begin();
			List artigos = this.selecionaArtigos();
			for (int i=0; i<artigos.size(); i++){
				Artigo artigo = ((Artigo)artigos.get(i));
				artigo.calculaNotaFinal();
				em.merge(artigo);
			}	
			em.getTransaction().commit();
	    	return artigos;	
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar artigos");
    	}

    }

    public void defineArtigosAceitos(String[] nomesArtigos){
    	try {
    		em.getTransaction().begin();
	    	for (int i=0; i<nomesArtigos.length;i++){
	    		Artigo artigo = em.find(Artigo.class,(String)nomesArtigos[i]);
	    		artigo.defineComoAceito();
	    		em.merge(artigo);
		    	}
	    	em.getTransaction().commit();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar artigos");
    	}
    }

    public void modificaStatusArtigo(String nomeArtigo){
    	try {
    		em.getTransaction().begin();
    		Artigo artigo = em.find(Artigo.class,nomeArtigo);
    		artigo.modificaStatus();
    		em.merge(artigo);
    		em.getTransaction().commit();
    	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao selecionar artigo");
    	}
    }
    
    public void termina(){
    	try {
			em.clear();
			factory.close();
   	} catch (Exception e) {
    		e.printStackTrace();
    		throw new java.lang.RuntimeException("erro ao fechar conexao");
    	}
    }

    @Id
    public String getNome(){
    	return this.nome;
    }
    
	public Date getData() {
		return data;
	}

	@Column(name="INICIO_SUB")
	public Date getDataInicioSubmissao() {
		return dataInicioSubmissao;
	}

	@Column(name="TERMINO_REV")
	public Date getDataTerminoRevisao() {
		return dataTerminoRevisao;
	}

	@Column(name="TERMINO_SUB")
	public Date getDataTerminoSubmissao() {
		return dataTerminoSubmissao;
	}

	public String getLocal() {
		return local;
	}

	@Column(name="MAX_ARTIGOS")
	public int getNumMaxArtigos() {
		return numMaxArtigos;
	}

	public void setData(Date data) {
		this.data = data;
	}

	public void setDataInicioSubmissao(Date dataInicioSubmissao) {
		this.dataInicioSubmissao = dataInicioSubmissao;
	}

	public void setDataTerminoRevisao(Date dataTerminoRevisao) {
		this.dataTerminoRevisao = dataTerminoRevisao;
	}

	public void setDataTerminoSubmissao(Date dataTerminoSubmissao) {
		this.dataTerminoSubmissao = dataTerminoSubmissao;
	}

	public void setLocal(String local) {
		this.local = local;
	}

	public void setNome(String nome) {
		this.nome = nome;
	}

	public void setNumMaxArtigos(int numMaxArtigos) {
		this.numMaxArtigos = numMaxArtigos;
	}

}


